<?php

require_once 'lib/controller/class.application.php' ;

/** Administration class for AvMP3 application
  *
  */
class admin extends application
{
	/** Get the road !
	  * Fetch the data access behind the path
	  * Got a display page to work on it
	  * Process data to act and got what to display
	  *
	  * @return boolean true|false true when display is afordable
	  */
	public function run()
	{
		$this->_source = MyAdventuresModel::getModel($this->_path) ;
		$this->_source->setFindableTypeList($this->_handledTypes) ;

		$this->_view = new AdventuresPage($this->_template) ;
		
		if($this->hasAction())
		{
			$success = $this->doAction() ;
			if($success)
			{
				$this->setActionParam('action', 'show') ;
			}

			$this->actionToRequest() ;
		}

		return $this->buildView() ;
	}

	protected function buildView()
	{
		$displayList = $this->getRequestParam('list') ;
		$type = $this->getRequestParam('type') ;

		if($this->isHandledType($type))
		{
			$method =  $displayList
				? "buildListView"
				: "buildSimpleView" ;
			$this->{$method}($type) 
				or $this->buildErrorView() ;

			$this->buildMenuView($type) ;
		}
		else
		{
			$this->buildDefaultView() ;
			$this->buildDefaultMenuView() ;
		}

	}

	protected function buildSimpleView($type)
	{
		if(!$this->hasRequestParam('id'))
			return $this->buildListView($type) ;

		$id = (integer) $this->getRequestParam('id') ;

		// $gettypebyid = "get{$type}ById" ;
		$thing = $this->_source->{$type}->getById($id) ;

		if(is_null($thing)) return false ;

		if($this->hasRequestParam('action'))
			$content = $this->_view->getContent
				( $type
				, false
				, $this->getRequestParam('action')) ;
		else
			$content = $this->_view->getContent($type, false) ;

		$content->setId('content') ;
		// @todo : config parameter
		$content->setScriptPathName('admin.php') ;
		$content->setEvent('ajouter', 'action', 'add') ;
		$content->setEvent('modifier', 'action', 'update') ;
		$content->setEvent('supprimer', 'action', 'delete') ;

		$settype = "set{$type}" ;
		$content->{$settype}($thing) ;
		
		switch($type)
		{
			case 'adventure' :
				$kinds = $this->_source->Kind->getThemAll() ;
				$content->setKindsList($kinds) ;
				break ;
			case 'newspaper' :
				$kinds = $this->_source->Kind->getThemAll() ;
				$content->setKindsList($kinds) ;
				$adventures = $this->_source->Adventure->getThemAll() ;
				$content->setAdventuresList($adventures) ;
		}

		return true ;
	}

	/*
	protected function buildUpdateView($type, $id)
	{
		$content->setId('content') ;
		$content->setScriptPathName('admin.php') ;
	}

	protected function buildDeleteView()
	{
		trigger_error('Unimplemented !') ;
	}
	*/

	protected function buildListView($type)
	{
		$offset = $this->hasRequestParam('offset')
			? (integer) $this->getRequestParam('offset')
			: $this->_source->{$type}->getFirstId($type) ;
		
		$thing_list = $this->_source->{$type}->getCollection($offset, 5) ; // @todo cfg num by list
		$thing_count = $this->_source->{$type}->getHowMany() ;

		$content = $this->_view->getContent($type, true) ;
		if(is_null($content))	return false ;

		$content->setId('content') ;
		$content->setScriptPathName('admin.php') ;
		$content->setAdventures($thing_list) ;
		$content->setRange($offset, 5) ;
		$content->setTotal($thing_count) ;
		$content->setScriptPathName('admin.php') ;
		$content->setEvent('ajouter', 'action', 'add') ;
		$content->setEvent('modifier', 'action', 'update') ;
		$content->setEvent('supprimer', 'action', 'delete') ;


		return true ;
	}

	protected function buildMenuView($type)
	{
		$menu = $this->_view->getMainMenu($type) ;
		$menu->setId('mainmenu') ;
	}

	protected function buildErrorView()
	{
		$this->_view->getError() ;
	}

	protected function buildDefaultView()
	{
		return $this->buildAdminView() ;
	}

	protected function buildDefaultMenuView()
	{
		trigger_error('non-implemented !') ;
	}

	protected function buildAdminView()
	{
		$wgadmin = $this->_view->getContent('admin') ;
		$wgadmin->setTypes($this->_handledTypes) ;
	}

	protected function doAction()
	{
		if(!$this->hasAction()) return null ;

		$action = $this->getAction() ;
		switch($action)
		{
			case 'add' :
			case 'update' :
			case 'delete' :
				$method = "do{$action}" ;
				return $this->{$method}() ;
			default:
				return false ;
		}
	}

	/**
	  */
	protected function extract()
	{
		switch($this->getActionParam('type'))
		{
			case 'kind' :
#				$extracted = new ExtractKind($post) ;

				$extracted = new Kind() ;
				$extracted->id = $this->getActionParam('id') ;
				$extracted->title = $this->getActionParam('title') ;
				$extracted->content = $this->getActionParam('content') ;

				break ;

			case 'adventure' :
#				$extracted = new ExtractAdventure( ;

				$extracted = new Adventure() ;
				$extracted->id = $this->getActionParam('id') ;
				$extracted->title = $this->getActionParam('title') ;
				$extracted->synopsis = $this->getActionParam('synopsis') ;
				$extracted->critic = $this->getActionParam('critic') ;
				$extracted->kind = new Kind($this->getActionParam('kind-id')) ;

				break ;

			case 'newspaper' :
				$extracted = new Newspaper() ;
				$extracted->id = $this->getActionParam('id') ;
				$extracted->title = $this->getActionParam('title') ;
				$extracted->content = $this->getActionParam('content') ;

				$extracted->adventure = new Adventure($this->getActionParam('adventure-id')) ;
				$extracted->kind = new Kind($this->getActionParam('kind-id')) ;

				break ;
				/*
			case 'author' :
#				$extracted = new ExtractAuthor($post) ;
				$author = new Author() ;
				return $this->_source->addAuthor($author) ;

			case 'cut' :
#				$extracted = new ExtractCut($post) ;
				$cut = new Cut() ;
				return $this->_source->addCut($cut) ;
				
			case 'news' :
#				$extracted = new ExtractNews($post) ;
				$news = new News() ;
				return $this->_source->addNews($news) ;
				*/
		}

		return $extracted ;
	}

	protected function doAdd()
	{
		if(!$this->hasActionParam('type'))
			return false ;

		$substract = $this->extract() ;
		
		$type = $this->getActionParam('type') ;
		if($this->isHandledType($type))
		{
			$method = "add{$type}" ;
			return $this->_source->{$method}($substract) ;
		}

		return false ;
	}

	protected function doUpdate()
	{
		if(!$this->hasActionParam('type'))
			return false ;

		$substract = $this->extract() ;
		
		$type = $this->getActionParam('type') ;
		if($this->isHandledType($type))
		{
			return $this->_source->{$type}->update($substract) ;
		}

		return false ;
	}

	protected function doDelete()
	{
		if(!$this->hasActionParam('type') or !$this->hasActionParam('id'))
			return false ;

		$substract = $this->extract() ;
		
		$type = $this->getActionParam('type') ;
		$id = (integer) $this->getActionParam('id') ;

		if($this->isHandledType($type))
		{
			$method = "delete{$type}ById" ;
			return $this->_model->{$method}($id) ;
		}

		return false ;
	}
}

?>
