<?php

abstract class application
{
	/** 
	  * @param string $path Path to data repository
	  * @param array $requestParams instruction on what to display
	  * @param array $post instructions on how to act
	  */
	public function __construct($cfg, $requestParams, $actionParams = null)
	{
		$this->_template = $cfg['template'] ;
		$this->setHandledTypes($cfg['types']) ;
		$this->setSource($cfg['path']) ;
		$this->setRequest($requestParams) ;
		$this->setAction($actionParams) ;
	}

	/** 
	  */
	abstract public function run() ;

	/** Flush the view content on the stdo !
	  */
	public function flush()
	{
		return $this->_view->display() ;
	}

	public function displayActionResult($success)
	{
		trigger_error('Unimplemented !') ;
	}

	public function setSource($path)
	{
		$this->_path = $path ;
	}

	public function setAction($actionParams)
	{
		$this->_actionParams = array() ;
		if(is_array($actionParams))
			foreach($actionParams as $key => $value)
				$this->_actionParams[$key] = $value ;
	}

	/** @todo check handeled action */
	public function hasAction()
	{
//		var_dump($this) ;
		return is_array(@$this->_actionParams)
			and array_key_exists('action', $this->_actionParams) ;
	}

	/** @todo get only handeled action */
	public function getAction()
	{
		return strtolower($this->_actionParams['action']) ;
	}

	/** @todo set only handeled action */
	public function setActionParam($key, $value)
	{
 		$this->_actionParams[$key] = $value ;
		return true ;
	}

	/** Check parameter present in {@link _actionParams}
	  *
	  * @todo Implement checking type using @inline{@see $type}
	  *
	  * @param string $name Param name
	  * @param string $type
	  *
	  * @return boolean
	  */
	public function hasActionParam($name, $type = null)
	{
		return is_array($this->_actionParams)
			and array_key_exists($name, $this->_actionParams) ;
	}

	/** Fetch parameter from {@link _actionParams}
	  *
	  * @todo Implement checking type using @inline{@see $type}
	  *
	  * @param string $name Param name
	  * @param string $type
	  *
	  * @return string Action parameter content
	  */
	public function getActionParam($name, $type = null)
	{
		return $this->_actionParams[$name] ;
	}

	public function setRequest($requestParams)
	{
		$this->_requestParams = array() ;
		if(is_array($requestParams))
			foreach($requestParams as $key => $value)
				$this->_requestParams[$key] = $value ;
	}

	/** @todo set only handeled action */
	public function setRequestParam($key, $value)
	{
 		$this->_requestParams[$key] = $value ;
		return true ;
	}

	/** Check parameter present in {@link _requestParams}
	  *
	  * @todo Implement checking type using @inline{@see $type}
	  *
	  * @param string $name Param name
	  * @param string $type
	  *
	  * @return boolean
	  */
	public function hasRequestParam($name, $type = null)
	{
		return is_array($this->_requestParams)
			and array_key_exists($name, $this->_requestParams) ;
	}

	/** Fetch parameter from {@link _requestParams}
	  *
	  * @todo Implement checking type using @inline{@see $type}
	  *
	  * @param string $name Param name
	  * @param string $type
	  *
	  * @return string Request parameter content
	  */
	public function getRequestParam($name, $type = null)
	{
		return $this->hasRequestParam($name)
			? strtolower($this->_requestParams[$name]) 
			: '' ;
	}

	public function setHandledTypes($aTypes)
	{
		$this->_handledTypes = $aTypes ;
	}

	public function isHandledType($type)
	{
		return in_array($type, $this->_handledTypes) ;
	}

	public function getHandledTypes()
	{
		$a = array() ;
		foreach($this->_handledTypes as $type)
			$a[] = $type ;

		return $a ;
	}
	
	/** Got the request fields and put it on the action array
	  */
	protected function requestToAction()
	{
		foreach($this->_requestParams as $key => $value)
			$this->_actionParams[$key] = $value ;

		unset($this->_requestParams) ;
	}

	/** Got the action fields and put it on the request array
	  */
	protected function actionToRequest()
	{
		foreach($this->_actionParams as $key => $value)
			$this->_requestParams[$key] = $value ;

		unset($this->_actionParams) ;
	}

	protected $_view = null ;
	protected $_source = null ;

	protected $_handledTypes = null ;

	protected $_path = '' ;
	protected $_template = '' ;

	private $_requestParams = null ;
	private $_actionParams = null ;
}

?>
