<?php

/**
  *
  */
interface Finder
{
	public function getById($id) ;
	public function getByName($name) ;
	public function getCollection($offset, $limit) ;
	public function getHowMany() ;
}

/** Define the common way to access to the data. It can derivate on
  * HTTP, MySQL or zipped XML files.
  * @todo extract generic model from this hell
  */
abstract class AdventuresModel
{
	abstract protected static function setModel($path) ;

	/** XXX: URIPattern is'nt static anymore !
	  * @return boolean results depend on handable URI
	  */
	static public function canHandleURI($uri)
	{
		return preg_match(self::$URIPattern, $uri) ;
	}

	/** Give a copy of path
	  * @return string The path
	  */
	public function getPath()
	{
		return $this->$_path ;
	}
	
	/** Make a connection to the {@link AdventuresModel::$_path}.
	  * @return boolean success status
	  */
	abstract protected function connect() ;

	/** Instantiate a data accessor to {@inline param $accessed}
	  * @param string $accessed Data name (type, table name, XMLnode name)
	  * @return boolean If some finder can be accessed...
	  */
	abstract protected function fetchFinder($accessed) ;
	
	/**
	  *
	  */
	public function __get($accessed)
	{
		$accessed = strtolower($accessed) ;
		return $this->canAccessData($accessed)
			? $this->_finders[$accessed]
			: $this->{$accessed} ;
	}

	public function addFinder($type)
	{
		if(in_array($type, $this->_findableTypeList))
			return false ;
		
		return $this->fetchFinder($type) ;
	}

	protected function canAccessData($accessed)
	{
		$accessed = strtolower($accessed) ;
		if(!in_array($accessed, $this->_findableTypeList))
		{
			return false ;
		}

		if(array_key_exists($accessed, $this->_findableTypeList))
		{
			return true ;
		}
		else
		{
			return $this->fetchFinder($accessed) ;
		}

		return false ;
	}

	/**
	  * @param string $class give the class name of the child to create
	  * 	if the ressource is not handle at this time
	  * @param string $path 
	  * @return AdventuresModel
	  */
	static public function getModel($class, $path)
	{
		if(array_key_exists($path, self::$_models))
		{
			return self::$_models[$path] ;
		}
		else
		{
			call_user_func_array(array($class, "setModel"), array($path)) ;
			return self::$_models[$path] ;
		}
	}

	/**
	  * @param string $path Path to ressource
	  * @return AdventuresModel|null
	  */
	public static function closeModel($path)
	{
		if(array_key_exists($path, self::$_models))
			unset(self::$_models[$path]) ;

		return null ;
	}

	/** Build the access object to the ressource, and connect to the
	  * ressource.
	  * @param string $path Path to ressource
	  */
	protected function __construct($path)
	{
		$this->_path = $path ;
		$this->connect() ;
	}

	/** Handler's stack. The key is the path to the ressource.
	  */
	protected static $_models = array() ;

	/** Path to the ressource handled by the current instance.
	  */
	protected $_path = '' ;

	protected $_findableTypeList = array() ;
	protected $_finders = array() ;
}

?>
