<?php

require_once 'model/class.adventuresmodel.php' ;
require_once 'model/class.adventure.php' ;

require_once 'tools/class.mytype.php' ;
require_once 'model/class.mykind.php' ;

class MyAdventureFinder extends MyFinder
{
	public function __construct($mycon)
	{
		parent::__construct($mycon) ;
		$this->_myclass = 'MyAdventure' ;
		$this->_tableName = 'Adventures' ;
		$this->_mycollectionclass = 'MyAdventuresCollection' ;
	}
}

class MyAdventuresCollection extends AdventuresCollection
{
	const SQL_SELECT = "select * from Adventures limit %d, %d" ;
	const SQL_SELECT_ALL = 'select * from Adventures' ;
	const SQL_COUNT = 'select count(*) as howmany from Adventures' ;

	public function __construct($collection = null)
	{
		parent::__construct($collection) ;
	}

	public function connect($handler)
	{
		$this->_handler = $handler ;
	}

	public function select($offset, $limit)
	{
		$sql = sprintf(self::SQL_SELECT, $offset, $limit) ;
		return $this->query($sql) and $this->fetch() ;
	}

	public function selectAll()
	{
		return $this->query(self::SQL_SELECT_ALL) and $this->fetch() ;
	}

	public function count()
	{
		$sql = sprintf(self::SQL_COUNT) ;
		$res = $this->_handler->query($sql) ;
		if($res)
		{
			$obj = $res->fetch_object() ;
			$this->count = $obj->howmany ;
		}
		else
			$this->count = 0 ;

		$res->free() ;

		return $this->count ;
	}

	public function getRaw()
	{
		return new AdventuresCollection($this) ;
	}
	
	protected function query($sql) 
	{
		$this->_result = $this->_handler->query($sql) ;
		return $this->_result instanceof mysqli_result;
	}

	protected function fetch()
	{
		if($this->_result instanceof mysqli_result 
				and ($this->_result->num_rows > 0))
		{
			while($obj = $this->_result->fetch_object())
			{
				$adv = new Adventure($obj) ;
				$this->push($adv) ;
			}

			$this->_result->free() ;

			return true ;
		}
		else
			return false ;
	}

	protected function setThingCollectionFromCollection($collection)
	{
		$this->_thingCollection = new AdventuresCollection($collection) ;
	}

	protected $count = null ;

	protected $_handler = null ;
	protected $_result = null ;
}

/** Manage MySQL access for the business object
  * @todo optimize code to load minimal datas instead all datas
  */
class MyAdventure extends MyType
{
	const SQL_FIND_FIRST = "select id from %s order by %s limit 1" ;
	const SQL_SELECT_BY_ID = "select title, synopsis, critic, kind_id from %s where id = ?" ;
	const SQL_SELECT_BY_NAME = "select id, synopsis, critic, kind_id from %s where title = ?" ;
	const SQL_INSERT = "insert %s (title, synopsis, critic, kind_id) values (?, ?, ?, ?)" ;
	const SQL_REPLACE = "replace %s (id, title, synopsis, critic, kind_id) values values (?, ?, ?, ?, ?)" ;
	const SQL_UPDATE = "update %s set title = ?, synopsis = ?, critic = ?, kind_id = ? where id = ?" ;
	const SQL_DELETE = "delete from %s where id = ?" ;

	const SQL_FIND_NAME = "select * from %s where title = ?" ;

	/** Call {@link MyType::__construct} and assign thing
	  * @param object|Type $param Data that permit to fetch informations
	  */
	public function __construct($param = null)
	{
		parent::__construct() ;
		$this->_thing = new Adventure($param) ;
		$this->_myTableName = 'Adventures' ;
	}

	protected function getSQLTemplate($method)
	{
		switch($method)
		{
			case 'FIND_FIRST'		: return self::SQL_FIND_FIRST ;
			case 'SELECT_BY_ID'		: return self::SQL_SELECT_BY_ID ;
			case 'SELECT_BY_NAME'	: return self::SQL_SELECT_BY_NAME ;
			case 'INSERT'			: return self::SQL_INSERT ;
			case 'REPLACE'			: return self::SQL_REPLACE ;
			case 'UPDATE'			: return self::SQL_DELETE ;
			default					: return 'no template' ;
		}
	}

	public function getRaw()
	{
		return new Adventure($this->_thing) ;
	}

	protected function setThingFromSQLStmt($stmt)
	{
		$stmt->bind_result
			( $title
			, $synopsis 
			, $critic
			, $kind_id
			) ;
		
		$stmt->execute() ;
		$stmt->store_result() ;
		$stmt->fetch() ;
		$stmt->free_result() ;
		$stmt->close() ;

		$this->_thing->title = $title ;
		$this->_thing->synopsis = $synopsis ;
		$this->_thing->critic = $critic ;

		$kind = new MyKind() ;
		$kind->connect($this->getHandler()) ;
		$kind->select($kind_id) ;
		$this->_thing->kind = $kind->getRaw() ;

		return true ;
	}

	protected function setSQLStmtFromThing($stmt)
	{
		$stmt->bind_param( 'issssi'
			, $this->_thing->id
			, $this->_thing->title
			, $this->_thing->synopsis 
			, $this->_thing->critic
			, $this->_thing->kind->id
			) ;
		$ret = ($stmt->execute() and $stmt->num_rows) ;
		$stmt->close() ;

		return $ret ;
	}

	protected function setThingFromStdObject($obj)
	{
		$this->_thing->id = $obj->id ;
		$this->_thing->title = $obj->title ;
		$this->_thing->synopsis  = $obj->synopsis ;
		$this->_thing->critic = $obj->critic ;
		$this->_thing->kind->id = $obj->kind_id ;
	}

	protected function setThingFromArray($array)
	{
		$this->_thing->id = $obj['id'] ;
		$this->_thing->title = $obj['title'] ;
		$this->_thing->synopsis  = $obj['synopsis'] ;
		$this->_thing->critic = $obj['critic'] ;
		$this->_thing->kind->id = $obj['kind_id'] ;
	}
}

?>
