<?php

require_once 'model/class.adventuresmodel.php' ;
require_once 'model/class.adventure.php' ;

class MyFinder implements Finder
{
	protected $_mycon = null ;
	protected $_myclass = '' ;
	protected $_mycollectionclass = '' ;

	public function __construct($model)
	{
		$this->_mycon = $model->getMyConnection() ;
	}
	
	protected function createMyThing($thing = null)
	{
		$myclass = "{$this->_myclass}" ;
		$my = new $myclass($thing) ;
		$my->connect($this->_mycon) ;

		return $my ;
	}

	protected function createMyCollection($things = null)
	{
		$myclass = "{$this->_mycollectionclass}" ;
		$mycol = new $myclass($things) ;
		$mycol->connect($this->_mycon) ;

		return $mycol ;
	}

	public function getFirstId()
	{
		$my = $this->createMyThing() ;
		$id = $my->getFirstId() ;

		unset($my) ;

		return $id ;
	}

	public function getById($id) 
	{
		$my = $this->createMyThing() ;
		$my->select($id) ;

		return $my->getRaw() ;
	}

	public function getByName($name)
	{
		$my = $this->createMyThing() ;
		$obj = $my->findByName($name) ;

		return $obj ;
	}

	public function update($thing)
	{
		$mything = $this->createMyThing($thing) ;
		return $mything->update() ;
	}
	
	public function getCollection($offset, $limit)
	{
		$mycol = $this->createMyCollection() ;

		if($mycol->select($offset, $limit))
			return $mycol->getRaw() ;
		else
			return null ;
	}

	public function getThemAll()
	{
		$mycol = $this->createMyCollection() ;

		if($mycol->selectAll())
			return $mycol->getRaw() ;
		else
			return null ;
	}

	public function getHowMany()
	{
		$mycol = $this->createMyCollection() ;
		$count = $mycol->count() ;
		unset($mycol) ;

		return $count ;
	}
	
	public function add($thing)
	{
		$my = $this->createMyThing($thing) ;
		$ret = $mything->add() ;

		unset($mything) ;

		return $ret ;
	}

	public function delete($thing)
	{
		$my = $this->createMyThing($thing) ;
		$ret = $mything->delete() ;

		unset($mything) ;

		return $ret ;
	}
}

require_once 'model/class.myadventure.php' ;
require_once 'model/class.mykind.php' ;
require_once 'model/class.mynewspaper.php' ;

class MyAdventuresModel extends AdventuresModel
{

	protected function __construct($path)
	{
		parent::__construct($path) ;
	}

	/** Get model on a ressource from path. If doesn't exists, we try to create it.
	  * @param string $path URI to ressource
	  * @return AdventuresModel|null Give model or null on error
	  */
	public static function getModel($path)
	{
		$model = parent::getModel(__CLASS__, $path) ;

		if(is_null($model))
			$model = new self($path) ;

		return $model ;
	}

	/** Set model on a ressource from path.
	  * @param string $path URI to ressource
	  * @return boolean false when ressource exists
	  */
	protected static function setModel($path)
	{
		if(array_key_exists($path, self::$_models))
			return false ;
		else
		{
			self::$_models[$path] = new self($path) ;
			return true ;
		}
	}

	protected function connect()
	{
		$pattern = '`^'.self::URIPattern.'$`' ;
		if(preg_match($pattern, $this->_path, $matches))
		{
			$path = $matches[0] ;
			if(strlen($matches[2]) == 0)
			{
				$user = substr($matches[1], 0, strpos($matches[1], '@')) ;
				$pass = '' ;
			}
			else
			{
				$user = substr($matches[1], 0, strpos($matches[1], ':')) ;
				$pass = substr($matches[2], 1, strlen($matches[2])) ;
			}
			$host = $matches[3] ;
			$port = $matches[4] ;

			if(count($matches) == 7)
			{
				$base = substr($matches[5], 1, strpos($matches[5], '/', 1) - 1 ) ;
				$this->prefix = substr($matches[7], 1, strlen($matches[7]) + 1) ;
			}
			else
			{
				$base = substr($matches[5], 1, strlen($matches[5]) + 1) ;
				$this->prefix = null ;
			}

			/* DEBUG
			$path = "mysql://$user:$pass@$host:$port/$base/{$this->prefix}" ;
			var_dump($matches, $path) ;
			*/

			$this->_handler = new mysqli($host , $user , $pass , $base) ;
		}

		if(mysqli_connect_errno())
		{
			trigger_error('No MySQL con !') ;
			return false ;
		}

		return true ;
	}

	public function getMyConnection()
	{
		return $this->_handler ;
	}

	public function __destruct()
	{
		$this->_handler->close() ;
	}
	
	protected function fetchFinder($accessed)
	{
		$accessed = strtolower($accessed) ;

		if(array_key_exists($accessed, $this->_finders))
			return true ;

		if(in_array($accessed, $this->_findableTypeList))
		{
			$myfinder = "My{$accessed}Finder" ;
			$this->_finders[$accessed] = new $myfinder($this) ;
		}

		return true ;
	}

	public function setFindableTypeList($aTypeList)
	{
		$this->_findableTypeList = $aTypeList ;
	}
	
	/** @param string Sometimes we can only access to one database namespace at once.
	 *  It this case, prefix is an ugly but comfortable workarround :)
	 * @todo implementing his handling :)
	 */
	private $_prefix = null ;

	/** @param mysqli Connector to database */
	private $_handler = null ;

	/** @param string Regex pattern (PCRE syntax) help us to validate a path and split it
	 *  into MySQL connection details.
	 *
	 *  A MySQL path looks like : mysql://user:pass@hostname:port/base/prefix
	 *  or with optionnal mentions, like : 'mysql://(user(:pass)?@)?(hostname)(:port)?(/base(/prefix)?)?'
	 *  And it can be defined as :
	 *    - mysql://
	 *    - optionnal user/password couple :
	 *      - user name, as a string begin by an alpha,
	 *        follow by alphanumeric or underscore
	 *      - semicolon if has password
	 *      - optionnal password, as a string begin by an alpha,
	 *      follow by alphanumeric or underscore
	 *      - finished by @
	 *    - host name
	 *    - semicolon followed by port number (integer between 1 and 65535)
	 *    - path designate base and prefix's table
	 *      - slash followed by /base
	 *      - /prefix
	 *
	 */
	const URIPattern =
		'mysql://(\w[\w\d_]+(:\w[\w\d_]+)?@)?(\w[\w\d.-]+\w)(:\d{1,5})?(/[\w\d_]+(/[\w\d_]+)?)?' ;
}

?>
