<?php

require_once 'model/class.adventuresmodel.php' ;
require_once 'model/class.adventure.php' ;
require_once 'model/class.newspaper.php' ;

require_once 'tools/class.mytype.php' ;
require_once 'model/class.mykind.php' ;
require_once 'model/class.myadventure.php' ;

class MyNewsPaperFinder extends MyFinder
{
	public function __construct($mycon)
	{
		parent::__construct($mycon) ;
		$this->_myclass = 'MyNewspaper' ;
		$this->_mycollectionclass = 'MyNewspapersCollection' ;
	}
}

class MyNewspapersCollection extends NewspapersCollection
{
	const SQL_SELECT = "select * from Newspapers limit %d, %d" ;
	const SQL_SELECT_ALL = 'select * from Newspapers' ;
	const SQL_COUNT = 'select count(*) as howmany from Newspapers' ;

	public function __construct($something = null)
	{
		parent::__construct($something) ;
	}

	public function connect($handler)
	{
		$this->_handler = $handler ;
	}

	public function select($offset, $limit)
	{
		$sql = sprintf(self::SQL_SELECT, $offset, $limit) ;
		return $this->query($sql) and $this->fetch() ;
	}

	public function count()
	{
		$sql = sprintf(self::SQL_COUNT) ;
		$res = $this->_handler->query($sql) ;
		if($res)
		{
			$obj = $res->fetch_object() ;
			$this->count = $obj->howmany ;
		}
		else
			$this->count = 0 ;

		$res->free() ;

		return $this->count ;
	}

	public function getRaw()
	{
		return new NewspapersCollection($this) ;
	}
	
	public function selectAll()
	{
		return $this->query(self::SQL_SELECT_ALL) and $this->fetch() ;
	}

	protected function query($sql) 
	{
		// echo "Query: $sql" ;

		$this->_result = $this->_handler->query($sql) ;
		return $this->_result instanceof mysqli_result;
	}

	protected function fetch()
	{
		if($this->_result instanceof mysqli_result 
				and ($this->_result->num_rows > 0))
		{
			while($obj = $this->_result->fetch_object())
			{
				$mynp = new MyNewsPaper($obj) ;
				$mynp->connect($this->_handler) ;
				$this->push($mynp->getRaw()) ;
			}

			$this->_result->free() ;

			return true ;
		}
		else
			return false ;
	}

	protected function setThingCollectionFromCollection($collection)
	{
		trigger_error('Unimplemented') ;
	}

	protected $count = null ;

	protected $_handler = null ;
	protected $_result = null ;
}

/** Manage MySQL access for the business object
  * @todo optimize code to load minimal datas instead all datas
  */
class MyNewspaper extends MyType
{
	const SQL_FIND_FIRST = "select id from %s order by %s limit 1" ;
	const SQL_SELECT_BY_ID = "select id, title, content, adventure_id, kind_id from %s where id = ?" ;
	const SQL_SELECT_BY_NAME = "select id, title, content, adventure_id, kind_id from %s where title = ?" ;
	const SQL_INSERT = "insert %s (id, title, content, adventure_id, kind_id) values (?, ?, ?, ?, ?)" ;
	const SQL_REPLACE = "replace %s (id, title, content, adventure_id, kind_id) values (?, ?, ?, ?, ?)" ;
	const SQL_UPDATE = "update %s set title = ?, content = ?, adventure_id = ?, kind_id = ? where id = ?" ;
	const SQL_DELETE = "delete from %s where id = ?" ;

	/** Call {@link MyType::__construct} and assign thing
	  * @param object|Type $param Data that permit to fetch informations
	  */
	public function __construct($param = null)
	{
		parent::__construct() ;
		$this->_thing = new Newspaper($param) ;
		$this->_myTableName = 'Newspapers' ;
	}

	protected function getSQLTemplate($method)
	{
		switch($method)
		{
			case 'FIND_FIRST'		: return self::SQL_FIND_FIRST ;
			case 'SELECT_BY_ID'		: return self::SQL_SELECT_BY_ID ;
			case 'SELECT_BY_NAME'	: return self::SQL_SELECT_BY_NAME ;
			case 'INSERT'			: return self::SQL_INSERT ;
			case 'REPLACE'			: return self::SQL_REPLACE ;
			case 'UPDATE'			: return self::SQL_UPDATE ;
			default					: return 'no template' ;
		}
	}

		/*
			) ;
			*/

	protected function setThingFromSQLStmt($stmt)
	{
		if($stmt->execute() and $stmt->num_rows > 0)
		{
			$stmt->store_result() ;
			$stmt->bind_result
				( $id
				, $title
				, $content 
				, $adventure_id
				, $kind_id
				) ;
			$stmt->fetch() ;
			$stmt->free_result() ;
			$stmt->close() ;

			$this->_thing->id = $id ;
			$this->_thing->title = $title ;
			$this->_thing->content = $content ;

			if(is_null($kind_id))
			{
				$this->_thing->kind = null ;
			}
			else
			{
				$kind = new MyKind() ;
				$kind->connect($this->getHandler()) ;
				$kind->select($kind_id) ;
				$this->_thing->kind = $kind->getRaw() ;
			}

			if(is_null($adventure_id))
			{
				$this->_thing->adventure = null ;
			}
			else
			{
				$adv = new MyAdventure() ;
				$adv->connect($this->getHandler()) ;
				$adv->select($adventure_id) ;
				$this->_thing->adventure = $adv->getRaw() ;
			}

			return true ;
		}
		else
		{
			$this->_thing = null ;
			return false ;
		}
	}

	protected function setSQLStmtFromThing($stmt)
	{
		$stmt->bind_param('ssiii'
			, $this->_thing->title
			, $this->_thing->content
			, $this->_thing->kind->id
			, $this->_thing->adventure->id
			, $this->_thing->id
			) ;
		$ret = ($stmt->execute() and $stmt->num_rows) ;
		$stmt->close() ;

		return $ret ;
	}

	protected function setThingFromStdObject($obj)
	{
		$this->_thing->id = $obj->id ;
		$this->_thing->title = $obj->title ;
		$this->_thing->synopsis  = $obj->synopsis ;
		$this->_thing->critic = $obj->critic ;
		$this->_thing->kind->id = $obj->kind_id ;
	}

	protected function setThingFromArray($array)
	{
		$this->_thing->id = $obj['id'] ;
		$this->_thing->title = $obj['title'] ;
		$this->_thing->synopsis  = $obj['synopsis'] ;
		$this->_thing->critic = $obj['critic'] ;
		$this->_thing->kind->id = $obj['kind_id'] ;
	}

	public function getRaw()
	{
		return new Newspaper($this->_thing) ;
	}

	protected function setThingFromSQLObject($obj)
	{
		$this->_thing->title = $obj->title ;
		$this->_thing->content = $obj->content ;

		$kind = new MyAdventure() ;
		$kind->connect($this->getHandler()) ;
		$kind->select($obj->adventure_id) ;
		$this->_thing->adventure = $kind->getRaw() ;

		$kind = new MyKind() ;
		$kind->connect($this->getHandler()) ;
		$kind->select($obj->kind_id) ;
		$this->_thing->kind = $kind->getRaw() ;
	}

}


?>
