<?php

require_once 'lib/tools/class.domdocext.php' ;

/** Main class that stock {@link WebGet WebGets} and encapsulate
  * DOMDocument through {@link DOMDocExt simplified object}.
  */
abstract class Page
{
	public function __construct()
	{
		$this->_webgets = array() ;
	}
	
	/** Create a named Webget and stores it in the {@link _webgets}
	  * array. 
	  * @param string $name key in {@link _webgets}
	  * @param string $type Webget type to create
	  * @param boolean $edit 
	  * @return created webget
	  */
	protected function createWebget($name, $type, $edit = false)
	{
		$wgclass = "WG$type" ;

		if(class_exists($wgclass))
		{
			if(!array_key_exists($name, $this->_webgets))
				$this->_webgets[$name] = new $wgclass($this->_canvas) ;
			else
				trigger_error("$name already in use !") ;
			
			$this->_webgets[$name]->setEditable($edit) ;
		}
		else
		{
			$this->_webget[$name] = null ;
			trigger_error("Trying to get a strange view '{$wgclass}' on the world !") ;
		}

		return $this->_webgets[$name] ;
	}

	/** Give a reference on named Webget object
	  * @param string $name
	  * @return object|null
	  */
	public function getWebget($name)
	{
		return array_key_exists($name, $this->_webgets)
			? $this->_webgets[$name] : null ;
	}

	/** Delete named {@link _webgets Webget}
	  * @param string $name
	  */
	public function deleteWebget($name)
	{
		if(array_key_exists($name, $this->_webgets))
			unset($this->_webgets[$name]) ;
	}

	/** Append each {@link _webgets} to {@link _canvas}
	  */
	protected function compile()
	{
		if(!$this->_compiled)
			foreach($this->_webgets as $id => $webget)
				$webget->append() ;

		$this->_compiled = true ;
	}

	/** Setting some useful header and flush {@link _canvas} string
	  * representation on stdout
	  */
	public function display()
	{
		$this->compile() ;

		header("Content-Type: {$this->_content_type} ;"
				." charset={$this->_canvas->encoding}") ;
#		header("Content-Length: ".strlen($page)) ;
		return $this->_canvas->save('php://output') ;
	}
	
	/** Saving {@link _canvas} string representation in a file
	  * @param string $file File to write
	  * @return boolean
	  */
	public function save($file)
	{
		$this->compile() ;
		return $this->_canvas->save($file) ;
	}
	
	public function __toString()
	{
		$this->compile() ;
		return $this->_canvas->saveXML() ;
	}

	/** Load template file
	  * @param string $file File to read
	  * @return boolean Validation status
	  */
	public function loadTemplate($file)
	{
		$this->_canvas = new DOMDocExt ;
		$this->_canvas->load($file) ;
		return $this->_canvas->validate() ;
	}

	/**
	  *
	  */
	protected $_content_type = 'text/html' ;

	/**
	  *
	  */
	protected $_canvas = null ;

	/**
	  *
	  */
	protected $_webgets = null ;

	/**
	  *
	  */
	protected $_model = null ;

	/**
	  *
	  */
	private $_compiled = false ;
}

?>
