<?php

require_once 'lib/tools/class.domdocext.php' ;

abstract class Webget
{
	abstract public function append() ;

	public function __construct(DomDocExt $canvas)
	{
		$this->_canvas = $canvas ;
	}

	public function setId($id)
	{
		$this->id = $id ;
		$this->_node = $this->_canvas->getElementById($id) ;
		if(is_null($this->_node))
			trigger_error('What id did you say ?') ;
	}

	public function getId($id)
	{
		return $this->id ;
	}
	
	public function setEditable($editable)
	{
		return $this->_edit = $editable ;
	}

	public function isEditable()
	{
		return $this->_edit ;
	}

	public function setScriptPathName($scriptpath)
	{
		$this->_scriptPathName = $scriptpath ;
	}

	public function setEvent($caption, $event, $value)
	{
		if(array_key_exists($event, $this->_events) and !is_array($this->_events[$event]))
			$this->_events[$event] = array() ;

		$this->_events[$event][] = array
			( 'caption' => $caption
			, 'value' => $value) ;
	}

	protected function createLinkEvent($event, $type, $id = null)
	{
		$url = new Url() ;
		$url->setFilename($this->_scriptPathName) ;
		if($event['value'] != 'add')	$url->setParam('id', $id) ;
		$url->setParam('type', $type) ;
		$url->setParam('action', $event['value']) ;

		$eventNode = $this->_canvas->createElement
			( 'a'
			, $event['caption']
			, array('href' => $url->get())
			) ;

		var_dump($url, $event) ;

		unset($url) ;
		return $eventNode ;
	}

	protected function createFormNode()
	{
		$formnode = $this->_canvas->createElement
			('form'
			, null
			, array
				( 'action' => "{$this->_scriptPathName}"
				, 'method' => 'post'
				)
			) ;

		return $formnode ;
	}

	protected function createActionInputNode()
	{
		$hidenode = $this->_canvas->createElement
			( 'input'
			, null
			, array
				( 'name' => 'action'
				, 'value' => 'update'
				, 'type' => 'hidden'
				)
			) ;
		
		return $hidenode ;
	}

	protected function createIdNode()
	{
		$idnode = $this->_canvas->createElement
			( 'input'
			, null
			, array
				( 'name' => 'id'
				, 'type' => 'hidden'
				, 'value' => "{$this->newspaper->id}"
				)
			) ;

		return $idnode ;
	}

	protected function createTypeNode()
	{
		$typenode = $this->_canvas->createElement
			( 'input'
			, null
			, array
				( 'name' => 'type'
				, 'type' => 'hidden'
				, 'value' => 'newspaper'
				)
			) ;
		return $typenode ;
	}

	protected function createLabeledFormInput($label_content, $input_id, $input_name, $input_content)
	{
		$labelnode = $this->_canvas->createElement
			( 'label'
			, "{$label_content}"
			, array('for' => "{$input_id}")
			) ;

		$spacer = $this->_canvas->createEntityReference('nbsp') ;
		$semicolon = $this->_canvas->createTextNode(':') ;

		$labelnode->appendChild($spacer) ;
		$labelnode->appendChild($semicolon) ;
			
		$inputnode = $this->_canvas->createElement
			( 'input'
			, null
			, array
				( 'name' => "{$input_name}"
				, 'value' => "{$input_content}"
				, 'id' => "{$input_id}"
				)
			) ;

		$caps = $this->_canvas->createElement('div') ;
		$caps->appendChild($labelnode) ;
		$caps->appendChild($inputnode) ;

		return $caps ;
	}

	protected function createLabeledFormTextarea($label_content, $input_id, $input_name, $input_content)
	{
		$labelnode = $this->_canvas->createElement
			( 'label'
			, "{$label_content}"
			, array('for' => "{$input_id}")
			) ;

		$spacer = $this->_canvas->createEntityReference('nbsp') ;
		$semicolon = $this->_canvas->createTextNode(':') ;

		$labelnode->appendChild($spacer) ;
		$labelnode->appendChild($semicolon) ;
			
		$inputnode = $this->_canvas->createElement
			( 'textarea'
			, "{$input_content}"
			, array
				( 'name' => "{$input_name}"
				, 'id' => "{$input_id}"
				, 'rows' => 10
				, 'cols' => 70
				)
			) ;

		$caps = $this->_canvas->createElement('div') ;
		$caps->appendChild($labelnode) ;
		$caps->appendChild($inputnode) ;

		return $caps ;
	}

	protected function createOptionList($thingName, $thingsList, $thingCurrent = null, $nullEntry = false)
	{
		$selectnode = $this->_canvas->createElement
			( 'select'
			, null
			, array('name' => "{$thingName}-id")
			) ;

		if(!is_null($thingsList))
		{
			foreach($thingsList as $thing)
			{
				$attrs = array('value' => "{$thing->id}") ;
				if(!is_null($thingCurrent)
					and $thingCurrent->id == $thing->id)
					$attrs['selected'] = 'selected' ;
	
				$optnode = $this->_canvas->createElement
					( 'option'
					, $thing->title
					, $attrs
					) ;
	
				$selectnode->appendChild($optnode) ;
			}
		}

		if($nullEntry)
		{
			$attrs = array('value' => 'null') ;
			if(is_null($thingCurrent))
				$attrs['selected'] = 'selected' ;
	
			$optnode = $this->_canvas->createElement('option', 'Aucun', $attrs) ;
			$selectnode->appendChild($optnode) ;
		}

		return $selectnode ;
	}

	protected $id = null ;

	protected $_canvas = null ;
	protected $_node = null ;
	protected $_edit = false ;
	protected $_scriptPathName = null ;
	protected $_events = array() ;
}

?>
